@@ -13,14 +13,10 @@ Tests for testing.tools
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
-from __future__ import with_statement
-from __future__ import print_function
 
 import os
 import unittest
 
-import nose.tools as nt
-
 from IPython.testing import decorators as dec
 from IPython.testing import tools as tt
 
@@ -30,26 +26,26 @@ from IPython.testing import tools as tt
 
 @dec.skip_win32
 def test_full_path_posix():
-    spath = '/foo/bar.py'
-    result = tt.full_path(spath,['a.txt','b.txt'])
-    nt.assert_equal(result, ['/foo/a.txt', '/foo/b.txt'])
-    spath = '/foo'
-    result = tt.full_path(spath,['a.txt','b.txt'])
-    nt.assert_equal(result, ['/a.txt', '/b.txt'])
-    result = tt.full_path(spath,'a.txt')
-    nt.assert_equal(result, ['/a.txt'])
+    spath = "/foo/bar.py"
+    result = tt.full_path(spath, ["a.txt", "b.txt"])
+    assert result, ["/foo/a.txt" == "/foo/b.txt"]
+    spath = "/foo"
+    result = tt.full_path(spath, ["a.txt", "b.txt"])
+    assert result, ["/a.txt" == "/b.txt"]
+    result = tt.full_path(spath, "a.txt")
+    assert result == ["/a.txt"]
 
 
 @dec.skip_if_not_win32
 def test_full_path_win32():
-    spath = 'c:\\foo\\bar.py'
-    result = tt.full_path(spath,['a.txt','b.txt'])
-    nt.assert_equal(result, ['c:\\foo\\a.txt', 'c:\\foo\\b.txt'])
-    spath = 'c:\\foo'
-    result = tt.full_path(spath,['a.txt','b.txt'])
-    nt.assert_equal(result, ['c:\\a.txt', 'c:\\b.txt'])
-    result = tt.full_path(spath,'a.txt')
-    nt.assert_equal(result, ['c:\\a.txt'])
+    spath = "c:\\foo\\bar.py"
+    result = tt.full_path(spath, ["a.txt", "b.txt"])
+    assert result, ["c:\\foo\\a.txt" == "c:\\foo\\b.txt"]
+    spath = "c:\\foo"
+    result = tt.full_path(spath, ["a.txt", "b.txt"])
+    assert result, ["c:\\a.txt" == "c:\\b.txt"]
+    result = tt.full_path(spath, "a.txt")
+    assert result == ["c:\\a.txt"]
 
     
 def test_parser():
@@ -58,18 +54,17 @@ def test_parser():
     both = ("FAILED (errors=1, failures=1)", 1, 1)
     for txt, nerr, nfail in [err, fail, both]:
         nerr1, nfail1 = tt.parse_test_output(txt)
-        nt.assert_equal(nerr, nerr1)
-        nt.assert_equal(nfail, nfail1)
+        assert nerr == nerr1
+        assert nfail == nfail1
 
 
 def test_temp_pyfile():
     src = 'pass\n'
-    fname, fh = tt.temp_pyfile(src)
+    fname = tt.temp_pyfile(src)
     assert os.path.isfile(fname)
-    fh.close()
-    with open(fname) as fh2:
+    with open(fname, encoding="utf-8") as fh2:
         src2 = fh2.read()
-    nt.assert_equal(src2, src)
+    assert src2 == src
 
 class TestAssertPrints(unittest.TestCase):
     def test_passing(self):
@@ -88,7 +83,7 @@ class TestAssertPrints(unittest.TestCase):
         self.assertRaises(AssertionError, func)
 
 
-class Test_ipexec_validate(unittest.TestCase, tt.TempFileMixin):
+class Test_ipexec_validate(tt.TempFileMixin):
     def test_main_path(self):
         """Test with only stdout results.
         """
@@ -110,8 +105,7 @@ class Test_ipexec_validate(unittest.TestCase, tt.TempFileMixin):
     def test_exception_path(self):
         """Test exception path in exception_validate.
         """
-        self.mktmp("from __future__ import print_function\n"
-                   "import sys\n"
+        self.mktmp("import sys\n"
                    "print('A')\n"
                    "print('B')\n"
                    "print('C', file=sys.stderr)\n"
@@ -123,8 +117,7 @@ class Test_ipexec_validate(unittest.TestCase, tt.TempFileMixin):
     def test_exception_path2(self):
         """Test exception path in exception_validate, expecting windows line endings.
         """
-        self.mktmp("from __future__ import print_function\n"
-                   "import sys\n"
+        self.mktmp("import sys\n"
                    "print('A')\n"
                    "print('B')\n"
                    "print('C', file=sys.stderr)\n"
